class SlackEvent(object):
    def __init__(self, sc=None, **kwargs):

        self._sc = sc
        self._channel = None
        self._user = None

        for k, v in kwargs.iteritems():
            setattr(self, k, v)

    @property
    def channel(self):
        return self._channel

    @channel.setter
    def channel(self, value):
        if isinstance(value, basestring):
            self._channel = SlackChannel(value, sc=self._sc)
        else:
            self._channel = value

    @property
    def user(self):
        return self._user

    @user.setter
    def user(self, value):
        if isinstance(value, basestring):
            self._user = SlackUser(value, sc=self._sc)
        else:
            self._user = value


class SlackUser(object):
    def __init__(self, userid, sc=None):
        self.userid = userid
        self._sc = sc
        self._username = None

    @property
    def username(self):
        if self._username is None and self._sc is not None:
            resp = self._sc.api_call('users.info', user=self.userid)
            self._username = resp['user']['name']
        return self._username

    def __str__(self):
        return '<@%s|%s>' % (self.userid, self.username)

    def __repr__(self):
        return self.userid


class SlackChannel(object):
    def __init__(self, channelid, sc=None):
        self.channelid = channelid
        self._sc = sc
        self._channel = None

    @property
    def channel(self):
        if self._channel is None and self._sc is not None:
            resp = self._sc.server.channels.find(self.channelid)
            self._channel = resp.name
        return self._channel

    def __str__(self):
        return '<#%s|%s>' % (self.channelid, self.channel)

    def __repr__(self):
        return self.channelid



from bottle import app
from rocket import Rocket


class Webserver(object):
    def __init__(self, host, port):
        self.host = host
        self.port = port
        self.webserver = None

    def start(self):
        interfaces = [(self.host, self.port)]
        self.webserver = Rocket(interfaces=interfaces, app_info={'wsgi_app': app()}, handle_signals=False)
        self.webserver.start(background=True)

    def stop(self):
        if self.webserver is not None:
            self.webserver.stop()



from slackminion.plugin.base import BasePlugin


class UserManager(BasePlugin):
    """
    Loads and stores user information
    """
    def on_load(self):

        self._dont_save = True  # Don't save this plugin's state on shutdown
        self.users = {}
        self.admins = {}
        if 'bot_admins' in self._bot.config:
            self.admins = self._bot.config['bot_admins']
        setattr(self._bot, 'user_manager', self)

        super(UserManager, self).on_load()

    def get(self, userid):
        """Retrieve user by userid"""
        if userid in self.users:
            return self.users[userid]
        return None

    def get_by_username(self, username):
        """Retrieve user by username"""
        res = filter(lambda x: x.username == username, self.users.values())
        if len(res) > 0:
            return res[0]
        return None

    def set(self, user):
        """
        Adds a user object to the user manager

        user - a SlackUser object
        """

        self.log.info("Loading user information for %s/%s", user.userid, user.username)
        self.load_user_info(user)
        self.log.info("Loading user rights for %s/%s", user.userid, user.username)
        self.load_user_rights(user)
        self.log.info("Added user: %s/%s", user.userid, user.username)
        self._add_user_to_cache(user)
        return user

    def _add_user_to_cache(self, user):
        self.users[user.userid] = user

    def load_user_info(self, user):
        """Loads additional user information and stores in user object"""
        # We have no additional information to load, but a child plugin
        # might want to override this
        pass

    def load_user_rights(self, user):
        """Sets permissions on user object"""
        if user.username in self.admins:
            user.is_admin = True



import logging
import threading


class BasePlugin(object):
    def __init__(self, bot, **kwargs):
        self.log = logging.getLogger(type(self).__name__)
        self._bot = bot
        self._dont_save = False  # By default, we want to save a plugin's state during save_state()
        self._state_handler = False  # State storage backends should set this to true
        self._timer_callbacks = {}
        self.config = {}
        if 'config' in kwargs:
            self.config = kwargs['config']

    def on_load(self):
        """
        Executes when a plugin is loaded.

        Override this if your plugin needs to do initialization when loading.
        Do not use this to restore runtime changes to variables -- they will be overwritten later on by PluginManager.load_state()
        """
        pass

    def on_unload(self):
        """
        (Not Implemented Yet) Executes when a plugin is unloaded.

        Override this if your plugin needs to do cleanup when unloading.
        """
        pass

    def on_connect(self):
        """
        Executes immediately after connecting to slack.

        Will not fire on reconnects.
        """
        pass

    def send_message(self, channel, text):
        """
        Used to send a message to the specified channel.

        * channel - can be a channel or user
        * text - message to send
        """
        self._bot.send_message(channel, text)

    def start_timer(self, duration, func, *args):
        """
        Schedules a function to be called after some period of time.

        * duration - time in seconds to wait before firing
        * func - function to be called
        * args - arguments to pass to the function
        """
        t = threading.Timer(duration, self._timer_callback, (func, args))
        self._timer_callbacks[func] = t
        t.start()
        self.log.info("Scheduled call to %s in %ds", func.__name__, duration)

    def stop_timer(self, func):
        """
        Stops a timer if it hasn't fired yet

        * func - the function passed in start_timer
        """
        if func in self._timer_callbacks:
            t = self._timer_callbacks[func]
            t.cancel()
            del self._timer_callbacks[func]

    def _timer_callback(self, func, args):
        del self._timer_callbacks[func]
        func(*args)



from setuptools import setup, find_packages
from slackminion.plugins.core import version

setup(
        name='slackminion',
        version=version,
        packages=find_packages(exclude=['test_plugins']),
        url='https://github.com/arcticfoxnv/slackminion',
        license='MIT',
        author='Nick King',
        author_email='',
        description='A python bot framework for slack',
        install_requires=[
            'bottle',
            'PyYAML',
            'redis',
            'requests',
            'Rocket',
            'six',
            'slackclient',
            'websocket-client',
        ],
        entry_points={
            'console_scripts': [
                'slackminion = slackminion.__main__:main',
            ]
        },
        classifiers=[
            "Programming Language :: Python",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
            "Development Status :: 2 - Pre-Alpha",
            "Topic :: Communications :: Chat",
        ]
)
